@@ -5,7 +5,7 @@
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
@@ -14,24 +14,14 @@
 ** and conditions see https://www.qt.io/terms-conditions. For further
 ** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** General Public License version 3 or (at your option) any later version
+** approved by the KDE Free Qt Foundation. The licenses are as published by
+** the Free Software Foundation and appearing in the file LICENSE.GPL3
 ** included in the packaging of this file. Please review the following
 ** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -45,12 +35,15 @@
 
 #include "qwaylandsurface.h"
 #include "qwaylandview.h"
-#include "qwaylandxkb_p.h"
 #include "qwaylandinputmethodeventbuilder_p.h"
 
 #include <QGuiApplication>
 #include <QInputMethodEvent>
 
+#if QT_CONFIG(xkbcommon)
+#include <QtXkbCommonSupport/private/qxkbcommon_p.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 QWaylandTextInputClientState::QWaylandTextInputClientState()
@@ -159,7 +152,7 @@ void QWaylandTextInputPrivate::sendInputMethodEvent(QInputMethodEvent *event)
     afterCommit.cursorPosition += event->commitString().length();
     afterCommit.anchorPosition = afterCommit.cursorPosition;
 
-    foreach (const QInputMethodEvent::Attribute &attribute, event->attributes()) {
+    for (const QInputMethodEvent::Attribute &attribute : event->attributes()) {
         if (attribute.type == QInputMethodEvent::Selection) {
             afterCommit.cursorPosition = attribute.start;
             afterCommit.anchorPosition = attribute.length;
@@ -171,7 +164,7 @@ void QWaylandTextInputPrivate::sendInputMethodEvent(QInputMethodEvent *event)
         }
     }
     send_commit_string(focusResource->handle, event->commitString());
-    foreach (const QInputMethodEvent::Attribute &attribute, event->attributes()) {
+    for (const QInputMethodEvent::Attribute &attribute : event->attributes()) {
         if (attribute.type == QInputMethodEvent::Cursor) {
             int index = QWaylandInputMethodEventBuilder::indexToWayland(event->preeditString(), attribute.start);
             send_preedit_cursor(focusResource->handle, index);
@@ -203,11 +196,15 @@ void QWaylandTextInputPrivate::sendKeyEvent(QKeyEvent *event)
 
     // TODO add support for modifiers
 
-    foreach (xkb_keysym_t keysym, QWaylandXkb::toKeysym(event)) {
+#if QT_CONFIG(xkbcommon)
+    for (xkb_keysym_t keysym : QXkbCommon::toKeysym(event)) {
         send_keysym(focusResource->handle, event->timestamp(), keysym,
                     event->type() == QEvent::KeyPress ? WL_KEYBOARD_KEY_STATE_PRESSED : WL_KEYBOARD_KEY_STATE_RELEASED,
                     0);
     }
+#else
+    Q_UNUSED(event);
+#endif
 }
 
 void QWaylandTextInputPrivate::sendInputPanelState()
@@ -605,3 +602,5 @@ QByteArray QWaylandTextInput::interfaceName()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qwaylandtextinput.cpp"
